"""
Classes and methods for laying the data location foundation for various types of CASA processing
"""
import logging
import os
import sys
import tarfile
from pathlib import Path

from casa_envoy.schema import AbstractTextFile

from casa_envoy.interfaces import FoundationIF


class RestoreFoundation(FoundationIF):
    def __init__(self, parameters: dict, metadata: AbstractTextFile):
        self.logger = logging.getLogger("casa_envoy")
        self.parameters = parameters
        self.parent_path = parameters["parent_path"]
        self.metadata = metadata

    def data_foundation(self):
        """
        CMS Restore requires two inputs: An EB and a Calibration for that EB
        After download, all data is in rawdata and the calibrations tables are contained
        in a tar file. We need to extract all the calibration tables to the products directory
        for CASA processing.
        :return:
        """

        self.logger.info("LAYING DATA FOUNDATION...")
        # ensure we're starting from the parent directory
        os.chdir(self.parent_path)
        self.extract_cal()
        self.set_permissions()
        self.logger.info("DATA FOUNDATION COMPLETE!")

    def extract_cal(self):
        self.logger.info("Extracting calibration tar file to products directory...")
        cal_name = self.metadata.content["fileSetIds"][1]
        cal_path = "./rawdata/" + cal_name + "/" + cal_name
        if Path(cal_path).exists():
            calibration = tarfile.open(cal_path)
            # extract to products
            calibration.extractall(path="./products")
            calibration.close()
        else:
            self.logger.error(f"ERROR: calibration tar file {cal_name} not found in rawdata!")
            # abort since required data not found
            sys.exit(1)

    def set_permissions(self):
        self.logger.info("Ensuring correct file permissions....")
        path = Path("./products")
        for root, dirs, files in os.walk(path):
            for d in dirs:
                os.chmod(os.path.join(root, d), 0o755)
            for f in files:
                os.chmod(os.path.join(root, f), 0o755)
