""" Tests for EVLA calibration ingestion manifest generation ONLY """

import json
import logging
import re
import shutil
import sys
from pathlib import Path

# pylint: disable=E0401, E0402, R0914

from ingest_envoy.ingestion_manifest import IngestionManifest
from ingest_envoy.ingestion_manifest_writer import (
    EvlaCalIngestionManifestWriter,
)
from ingest_envoy.utilities import (
    WEBLOG,
    ScienceProductType,
    MANIFEST_NAME_BASE,
    MANIFEST_NAME_EXT,
    EvlaCalOutputScienceProduct,
    Weblog,
    EvlaCalIngestionManifest,
    IngestionManifestKey,
)
from .conftest import (
    find_example_manifest,
    populate_fake_ingest_path,
    WANTED_FILENAMES,
    UNWANTED,
)

logger = logging.getLogger(IngestionManifest.__name__)
logger.setLevel(logging.INFO)
logger.addHandler(logging.StreamHandler(sys.stdout))


# pylint: disable=E0402, R1721


def test_creates_empty_evla_cal_manifest(ingest_path: Path):
    """
    Make sure we create the manifest we expect.

    :param ingest_path: ingestion location
    :return:
    """
    manifest = EvlaCalIngestionManifest(ingest_path, "")

    sps_in = manifest.input_group.science_products
    assert len(sps_in) == 1
    sp_in = sps_in[0]
    assert sp_in.type == ScienceProductType.EVLA_CAL
    assert sp_in.locator == ""

    outgrp = manifest.output_group
    sps_out = outgrp.science_products
    assert len(sps_out) == 1
    sp_out = sps_out[0]
    assert sp_out.type == sp_in.type

    weblogs = outgrp.ancillary_products
    assert len(weblogs) == 1
    weblog = weblogs[0]
    assert isinstance(weblog, Weblog)
    ap_out = weblog.ancillary_product
    assert ap_out["type"] == "weblog"
    assert ap_out["filename"] == "None"

    ingest_path.rmdir()


def test_creates_expected_manifest(ingest_path: Path):
    """
    Make sure we create the manifest we expect.

    :param ingest_path: ingestion location
    :return:
    """

    locator = "uid://evla/calibration/abcdefg_2_bubble_bubble"
    all_files = populate_fake_ingest_path(ingest_path)
    manifest = EvlaCalIngestionManifest(ingest_path, locator)
    sps_in = manifest.input_group.science_products
    assert len(sps_in) == 1
    sp_in = sps_in[0]
    assert sp_in.type == ScienceProductType.EVLA_CAL
    assert sp_in.locator == locator

    outgrp = manifest.output_group
    sps_out = outgrp.science_products
    assert len(sps_out) == 1
    sp_out = sps_out[0]
    assert isinstance(sp_out, EvlaCalOutputScienceProduct)
    assert sp_out.type == sp_in.type

    files_after = [file for file in ingest_path.iterdir()]
    # nothing should have been removed from staging dir
    assert len(files_after) == len(all_files)
    assert Path(sp_out.filename) in files_after

    aps = outgrp.ancillary_products
    assert len(aps) == 1
    anc_prod = aps[0]
    assert isinstance(anc_prod, Weblog)
    assert Path(ingest_path / WEBLOG).exists()

    shutil.rmtree(ingest_path)


def test_writes_real_manifest_to_file(ingest_path: Path):
    """
    We should get a pretty, formatted, human-readable JSON text file

    :param ingest_path: the staging dir
    :return:
    """

    populate_fake_ingest_path(ingest_path)

    writer = EvlaCalIngestionManifestWriter(ingest_path=ingest_path)
    locator = "uid://evla/calibration/my_devastating_observation"
    manifest_file, _ = writer.write_evla_cal_manifest(locator)

    with open(manifest_file, "r") as mf_in:
        manifest_content = dict(json.load(mf_in).items())

    assert len(manifest_content.keys()) >= len(IngestionManifestKey) - 1
    for key in ["parameters", "input_group", "output_group"]:
        assert key in manifest_content.keys()

    shutil.rmtree(ingest_path)


def test_builds_expected_manifest_filename(ingest_path: Path):
    """
    We expect the manifest to be named like

    ingestion_manifest_2019_07_30_T13_03_00.936.json

    :param ingest_path: ingestion location
    :return:
    """

    filename = EvlaCalIngestionManifestWriter.manifest_filename()

    assert filename.startswith(MANIFEST_NAME_BASE)
    assert filename.endswith(MANIFEST_NAME_EXT)

    filename_parts = filename.split("_")
    assert len(filename_parts) == 7

    # get just the timestamp
    timestamp = filename.replace(MANIFEST_NAME_BASE, "").replace(MANIFEST_NAME_EXT, "")

    # we should have gotten year, month, day, hours, minutes, seconds to 3 decimal places
    assert re.match(r"\d{4}_\d{2}_\d{2}T\d{2}_\d{2}_\d{2}\.\d{0,3}", timestamp)

    shutil.rmtree(ingest_path)


def test_writes_expected_output_files(ingest_path: Path):
    """
    We expect to get the manifest file, the weblog, and the science product tar.

    :param ingest_path: ingestion location
    :return:
    """

    populate_fake_ingest_path(ingest_path)

    example_manifest = find_example_manifest("evla_calibration")
    content = ""
    with open(example_manifest, "r") as infile:
        for line in infile.readlines():
            content += line
    mf_json = json.loads(content)

    ig_in = mf_json["input-group"]
    ig_sp = ig_in["science_products"][0]

    ip_in = mf_json["parameters"]["ingestion_path"]
    assert ip_in == "/home/mchammer/evla/parallel-prod"

    writer = EvlaCalIngestionManifestWriter(ingest_path=ingest_path)
    manifest_file, more_ingestion_files = writer.write_evla_cal_manifest(ig_sp["locator"])
    assert manifest_file.exists()
    for file in more_ingestion_files:
        assert file.exists()
    assert len(more_ingestion_files) == 1

    # make sure that one file is the artifacts tar
    file = more_ingestion_files[0]
    assert file.exists()
    assert file.name.startswith("ingestion_artifacts") and file.name.endswith(".tar")

    shutil.rmtree(ingest_path)


def test_filters_cal_input_files(ingest_path: Path):
    """
    We'll be getting calibration products from a directory under
    /lustre/aoc/cluster/pipeline/{CAPO_PROFILE}/workspaces/staging

    Make sure we take -only- the files to be ingested.

    :param ingest_path: our temporary dir
    :return:
    """
    populate_fake_ingest_path(ingest_path)

    writer = EvlaCalIngestionManifestWriter(ingest_path=ingest_path)
    locator = "uid://evla/calibration/im_a_one-touch_espresso_machine"
    manifest_file, _ = writer.write_evla_cal_manifest(locator)

    with open(manifest_file, "r") as mf_in:
        manifest_content = dict(json.load(mf_in).items())
    input_group = manifest_content[IngestionManifestKey.INPUT_GROUP.value]
    assert isinstance(input_group, dict)
    assert len(input_group) == 1

    for val in input_group.values():
        sci_prod = val.replace("'", '"', len(val))
        sps = json.loads(sci_prod)

    assert isinstance(sps, list)
    assert len(sps) == 1
    sci_prod = sps[0]
    assert sci_prod["type"] == ScienceProductType.EVLA_CAL.value
    assert sci_prod["locator"] == locator

    output_group = manifest_content[IngestionManifestKey.OUTPUT_GROUP.value]
    assert isinstance(output_group, dict)
    assert len(output_group) == 2

    for key, val in output_group.items():
        if key == IngestionManifestKey.SCIENCE_PRODUCTS.value:
            sci_prod = val.replace("'", '"', len(val))
            sps = json.loads(sci_prod)
            assert len(sps) == 1
        else:
            assert key == IngestionManifestKey.ANCILLARY_PRODUCTS.value
            anc_prod = val.replace("'", '"', len(val))
            aps = json.loads(anc_prod)
            assert len(aps) == 1

    for sci_prod in sps:
        filename = Path(sci_prod["filename"]).name
        file = Path(ingest_path / filename)
        assert file.exists()
        assert file.parent == ingest_path
        assert filename in WANTED_FILENAMES
        assert filename not in UNWANTED

    anc_prod_dict1 = aps[0]
    for key, val in anc_prod_dict1.items():
        anc_prod_dict2 = val

    assert isinstance(anc_prod_dict2, dict)
    for key, val in anc_prod_dict2.items():
        if key == "type":
            assert val == "weblog"
        else:
            assert key == "filename"
            file = Path(ingest_path / val)
            assert file.exists()

    shutil.rmtree(ingest_path)
